@@ -25,6 +25,8 @@ from yellowdog_cli.utils.entity_utils import (
     find_compute_source_template_id_by_name,
     get_application_id_by_name,
     get_group_id_by_name,
+    get_namespace_id_by_name,
+    get_worker_pools,
     remove_allowances_matching_description,
 )
 from yellowdog_cli.utils.interactive import confirmed
@@ -37,16 +39,18 @@ from yellowdog_cli.utils.settings import (
     RN_APPLICATION,
     RN_CONFIGURED_POOL,
     RN_CREDENTIAL,
+    RN_EXTERNAL_USER,
     RN_GROUP,
     RN_IMAGE_FAMILY,
+    RN_INTERNAL_USER,
     RN_KEYRING,
+    RN_NAMESPACE,
     RN_NAMESPACE_POLICY,
     RN_NUMERIC_ATTRIBUTE_DEFINITION,
     RN_REQUIREMENT_TEMPLATE,
     RN_SOURCE_TEMPLATE,
     RN_STORAGE_CONFIGURATION,
     RN_STRING_ATTRIBUTE_DEFINITION,
-    RN_USER,
 )
 from yellowdog_cli.utils.wrapper import ARGS_PARSER, CLIENT, CONFIG_COMMON, main_wrapper
 from yellowdog_cli.utils.ydid_utils import YDIDType, get_ydid_type
@@ -117,10 +121,12 @@ def remove_resources(resources: Optional[List[Dict]] = None):
                 remove_group(resource)
             elif resource_type == RN_APPLICATION:
                 remove_application(resource)
-            elif resource_type == RN_USER:
+            elif resource_type in [RN_INTERNAL_USER, RN_EXTERNAL_USER]:
                 print_warning(
                     "Users cannot be removed by the CLI; please use the YellowDog Portal"
                 )
+            elif resource_type == RN_NAMESPACE:
+                remove_namespace(resource)
             else:
                 print_error(f"Unknown resource type '{resource_type}'")
         except Exception as e:
@@ -201,17 +207,17 @@ def remove_keyring(resource: Dict):
         print_error(f"Expected property to be defined ({e})")
         return
 
-    if not confirmed(f"Delete Keyring '{name}'?"):
+    if not confirmed(f"Remove Keyring '{name}'?"):
         return
 
     try:
         CLIENT.keyring_client.delete_keyring_by_name(name)
-        print_log(f"Deleted Keyring '{name}'")
+        print_log(f"Removed Keyring '{name}'")
     except HTTPError as e:
         if e.response.status_code == 404:
-            print_error(f"Keyring '{name}' not found")
+            print_error(f"Cannot find Keyring '{name}'")
         else:
-            print_error(f"Unable to delete Keyring '{name}': {e}")
+            print_error(f"Unable to remove Keyring '{name}': {e}")
 
 
 def remove_credential(resource: Dict):
@@ -240,7 +246,7 @@ def remove_credential(resource: Dict):
     except HTTPError as e:
         if e.response.status_code == 404:
             print_error(
-                f"Keyring '{keyring_name}' not found (possibly already deleted,"
+                f"Cannot find Keyring '{keyring_name}'(possibly already deleted,"
                 " including its credentials?)"
             )
         else:
@@ -269,7 +275,7 @@ def remove_image_family(resource: Dict):
         )
     except HTTPError as e:
         if e.response.status_code == 404:
-            print_error(f"Machine Image Family '{fq_name}' not found")
+            print_error(f"Cannot find Machine Image Family '{fq_name}'")
             return
         else:
             raise e
@@ -279,9 +285,9 @@ def remove_image_family(resource: Dict):
 
     try:
         CLIENT.images_client.delete_image_family(image_family)
-        print_log(f"Deleted Image Family '{fq_name}' ('{image_family.id}')")
+        print_log(f"Removed Image Family '{fq_name}' ({image_family.id})")
     except Exception as e:
-        print_error(f"Unable to delete Image Family '{fq_name}': {e}")
+        print_error(f"Unable to remove Image Family '{fq_name}': {e}")
 
 
 def remove_namespace_configuration(resource: Dict):
@@ -298,7 +304,7 @@ def remove_namespace_configuration(resource: Dict):
         CLIENT.object_store_client.get_namespace_storage_configurations()
     )
     if namespace not in [x.namespace for x in namespaces]:
-        print_error(f"Namespace Storage Configuration '{namespace}' not found")
+        print_error(f"Cannot find Namespace Storage Configuration '{namespace}'")
         return
 
     if not confirmed(f"Remove Namespace Storage Configuration '{namespace}'?"):
@@ -326,17 +332,13 @@ def remove_configured_worker_pool(resource: Dict):
 
     fq_name = f"{namespace}{NAMESPACE_PREFIX_SEPARATOR}{name}"
 
-    worker_pools: List[WorkerPoolSummary] = (
-        CLIENT.worker_pool_client.find_all_worker_pools()
+    worker_pools: List[WorkerPoolSummary] = get_worker_pools(
+        CLIENT, CONFIG_COMMON.namespace, CONFIG_COMMON.name_tag
     )
 
     # Shut down a matching Configured Worker Pool if in an appropriate state
     for worker_pool in worker_pools:
-        if (
-            worker_pool.name == name
-            and worker_pool.namespace == namespace
-            and worker_pool.type.split(".")[-1] == "ConfiguredWorkerPool"
-        ):
+        if worker_pool.type.split(".")[-1] == "ConfiguredWorkerPool":
             if worker_pool.status not in [
                 WorkerPoolStatus.SHUTDOWN,
                 WorkerPoolStatus.TERMINATED,
@@ -426,7 +428,7 @@ def remove_resource_by_id(resource_id: str):
                         CLIENT.keyring_client.delete_keyring_by_name(keyring.name)
                         print_log(f"Removed Keyring {resource_id}")
                         return
-                raise Exception(f"Keyring {resource_id} not found")
+                raise Exception(f"Cannot find Keyring {resource_id}")
 
         elif get_ydid_type(resource_id) == YDIDType.WORKER_POOL:
             if confirmed(f"Shut down Worker Pool {resource_id}?"):
@@ -495,7 +497,7 @@ def remove_namespace_policy(resource: Dict):
             return
     except Exception:
         # Assume it's not found ... 404 from API
-        print_error(f"Namespace Policy '{namespace}' not found")
+        print_error(f"Cannot find Namespace Policy '{namespace}'")
         return
 
     if not confirmed(f"Remove Namespace Policy '{namespace}'?"):
@@ -520,7 +522,7 @@ def remove_group(resource: Dict):
 
     group_id = get_group_id_by_name(CLIENT, group_name)
     if group_id is None:
-        print_warning(f"Group '{group_name}' not found")
+        print_warning(f"Cannot find Group '{group_name}'")
         return
 
     if not confirmed(f"Remove Group '{group_name}' ({group_id})?"):
@@ -531,7 +533,7 @@ def remove_group(resource: Dict):
         print_log(f"Removed Group '{group_name}' ({group_id})")
         clear_group_caches()
     except Exception as e:
-        print_error(f"Unable to delete Group '{group_name}' ({group_id}): {e}")
+        print_error(f"Unable to remove Group '{group_name}' ({group_id}): {e}")
 
 
 def remove_application(resource: Dict):
@@ -546,7 +548,7 @@ def remove_application(resource: Dict):
 
     app_id = get_application_id_by_name(CLIENT, app_name)
     if app_id is None:
-        print_warning(f"Application '{app_name}' not found")
+        print_warning(f"Cannot find Application '{app_name}'")
         return
 
     if not confirmed(f"Remove Application '{app_name}' ({app_id})?"):
@@ -560,6 +562,40 @@ def remove_application(resource: Dict):
         print_error(f"Unable to remove Application '{app_name}' ({app_id}): {e}")
 
 
+def remove_namespace(resource: Dict):
+    """
+    Remove a namespace.
+    """
+    try:
+        name = resource["name"]
+    except KeyError as e:
+        print_error(f"Expected property to be defined ({e})")
+        return
+
+    namespace_id = get_namespace_id_by_name(CLIENT, name)
+    if namespace_id is None:
+        print_warning(f"Cannot find Namespace '{name}'")
+        return
+
+    if not confirmed(f"Remove Namespace '{name}'?"):
+        return
+
+    try:
+        CLIENT.namespaces_client.delete_namespace(
+            get_namespace_id_by_name(CLIENT, name)
+        )
+    except Exception as e:
+        if "ConflictException" in str(e):
+            print_error(
+                f"Unable to remove Namespace '{name}'; note: Namespaces that "
+                f"have been populated cannot currently be removed"
+            )
+        else:
+            print_error(f"Unable to remove Namespace '{name}': {e}")
+
+    print_log(f"Removed Namespace '{name}' ({namespace_id})")
+
+
 # Entry point
 if __name__ == "__main__":
     main()
